#ifndef CREATURELIB_ITEM_HPP
#define CREATURELIB_ITEM_HPP

#include "../Attacks/SecondaryEffect.hpp"
#include "BattleItemCategory.hpp"
#include "ItemCategory.hpp"

namespace CreatureLib::Library {
    class Item {
    private:
        struct impl;
        std::unique_ptr<impl> _impl;

    public:
        Item(const ArbUt::StringView& name, ItemCategory category, BattleItemCategory battleCategory, int32_t price,
             const SecondaryEffect* effect, const std::unordered_set<uint32_t>& flags) noexcept;
        NO_COPY_OR_MOVE(Item)

        virtual ~Item();

        const ArbUt::StringView& GetName() const noexcept;
        ItemCategory GetCategory() const noexcept;
        BattleItemCategory GetBattleCategory() const noexcept;
        int32_t GetPrice() const noexcept;

        const ArbUt::OptionalUniquePtr<const SecondaryEffect>& GetEffect() const noexcept;
        bool HasFlag(const ArbUt::BasicStringView& flag) const noexcept;
        bool HasFlag(uint32_t flag) const noexcept;
    };
}

#endif // CREATURELIB_ITEM_HPP
